% generateProblemData.m
%
% Matlab Script to generate a random matrix P of dimension 500 x 500 that 
% represents a shortest path problem.  The (i,j)-th element of the matrix 
% P denotes the cost of the transition from node i to the node j. Each 
% element is an integer in the range from 0 to 10000. 
% The matrix P is saved in the file problemData.mat.
%
% Dynamic Programming and Optimal Control
% Fall 2009
% Programming Exercise 1
%
% --
% ETH Zurich
% Institute for Dynamic Systems and Control
% Sebastian Trimpe
% strimpe@ethz.ch
%
% --
% Revision history
% [20.10.09, ST]    first version
%
%


% clear variable P that will hold the problem data
clear P;


%% Generate random matrix P
% The entries are chosen such that the closer two nodes i and j are in
% terms of the number (i.e. the smaller |i-j|) a smaller distance is more
% likely.

% Total number of nodes
M = 500;

% Range of values of P
up = 10000;
low = 0;

P = zeros(M,M);

% Loop though each element of the matrix.
for i=1:M
    for j=1:M
        P(i,j) = ceil(norm(i-j)/M*(up-low)*rand);
    end;
end;


%% 3D plot of elements of P
[I,J] = meshgrid(1:M);
mesh(I,J,P);
xlabel('i');
ylabel('j');
zlabel('p_{ij}');


%% Save data in mat file
save problemData.mat P;
