% script.m
%
% Matlab Script that is used to load the problem data, execute the shortest
% path algorithms and display the results.
%
% Dynamic Programming and Optimal Control
% Fall 2009
% Programming Exercise 1
%
% --
% ETH Zurich
% Institute for Dynamic Systems and Control
% Sebastian Trimpe
% strimpe@ethz.ch
%
% --
% Revision history
% [20.10.09, ST]    first version
%
%


% clear workspace and command window
clear;
clc;


%% Read in problem data
% Load Problem matrix P from mat file.  Element (i,j) of the matrix P is
% the cost of getting from node i to node j.
% Uncomment the line of the problem you want to solve.
load origProblemData.mat;
%load exampleProblemData.mat;   
%load problemData.mat;

% Do some sanity checks with matrix P:
% Is P a square matrix?
[r,c] = size(P);
if r~=c
    error('Problem matrix is not square.');
end;

% Does P have negative entries?
if any(any(P<0))
    error('Problem matrix has negative entries.');
end;

% Total number of nodes
totNodes = length(P);



%% Choose start and terminal node
% User Input of start and terminal node.  Both start and terminal node can 
% take values between 1 and the total number of nodes.

disp('------------------------------------------------------------------');
disp('Shortest Path Problem'); disp(' ');

% User Input: Start node
inpStartNode = input(['Enter start node (',num2str(1),'..',num2str(totNodes),'): ']);

% Check user input
if isempty(inpStartNode)
    inpStartNode = 1;
    disp(['No start node specified.  Use default start node: ',num2str(inpStartNode),'.']);
end; 
if ~isnumeric(inpStartNode) || inpStartNode > totNodes
    inpStartNode = 1;
    disp(['Input start node not valid.  Use default start node: ',num2str(inpStartNode),'.']);
end;
startNode = inpStartNode;       % store for later use

% User Input: Terminal node
inpTermNode = input(['Enter terminal node (',num2str(1),'..',num2str(totNodes),'): ']);

% Check user input
if isempty(inpTermNode)
    inpTermNode = totNodes;
    disp(['No Terminal node specified.  Use default terminal node: ',num2str(inpTermNode),'.']);
end; 
if ~isnumeric(inpTermNode) || inpTermNode > totNodes
    inpTermNode = totNodes;
    disp(['Input terminal node not valid.  Use default terminal node: ',num2str(inpTermNode),'.']);
end;
termNode = inpTermNode;




%% Solution Method 1: Dynamic Programming Algorithm
% Solve Shortest Path (SP) problem using the Dynamic Programming Algorithm
% (DPA).
% Input: problem data P, start Node, terminal Node
% Output: optimal path length, optimal path
% Use tic, toc to time the execution.
tic;
[optCost1, optPath1] = sp_dpa(P,startNode,termNode);
time1 = toc;

    

%% Solution Method 2: Label Correcting Algorithm
% Solve Shortest Path (SP) problem using the Label Correcting Algorithm
% (DPA) with Dijkstra's method.
tic;
[optCost2, optPath2] = sp_lca(P,startNode,termNode);
time2 = toc;





%% Display results
disp(' ');
disp('------------------------------------------------------------------');
disp('Results');
disp(['Problem with ',num2str(totNodes),' nodes.']);
disp(['Optimal path from node ',num2str(inpStartNode),' to ',num2str(inpTermNode),':']);
disp('------------------------------------------------------------------');
disp(' ');
disp('Method 1: Dynamic Programming Algorithm');
disp(['  Execution time: ',num2str(time1),'s.']);
disp(['  Minimum path length (minimum total cost): ',num2str(optCost1)]);
disp(['  Path: ',num2str(optPath1)]);
disp(' ');
disp('Method 2: Label Correcting Algorithm');
disp(['  Execution time: ',num2str(time2),'s  (',num2str(time2/time1),' times the time for method 1).']);
disp(['  Minimum path length (minimum total cost): ',num2str(optCost2)]);
disp(['  Path: ',num2str(optPath2)]);
disp(' ');
disp('------------------------------------------------------------------');
            
        
    


    


